@@ -1,8 +1,8 @@
 /* database.c - deals with database subsystem */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/database.c,v 1.61.2.9 2007/01/02 21:44:04 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/database.c,v 1.61.2.12 2008/05/26 18:56:42 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2007 The OpenLDAP Foundation.
+ * Copyright 2001-2008 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -49,14 +49,6 @@ static int monitor_back_add_plugin( monitor_info_t *mi, Backend *be, Entry *e );
 #include "../back-meta/back-meta.h"
 #endif /* defined(SLAPD_META) */
 
-/* for PATH_MAX on some systems (e.g. Solaris) */
-#ifdef HAVE_LIMITS_H
-#include <limits.h>
-#endif /* HAVE_LIMITS_H */
-#ifndef PATH_MAX
-#define PATH_MAX	4095
-#endif /* ! PATH_MAX */
-
 static int
 monitor_subsys_database_modify(
 	Operation	*op,
@@ -313,7 +305,7 @@ monitor_subsys_database_init(
 		{
 			struct berval	bv;
 			ber_len_t	pathlen = 0, len = 0;
-			char		path[ PATH_MAX ] = { '\0' };
+			char		path[ MAXPATHLEN ] = { '\0' };
 			struct bdb_info *bdb = (struct bdb_info *) be->be_private;
 			char		*fname = bdb->bi_dbenv_home;
 
@@ -383,7 +375,7 @@ monitor_subsys_database_init(
 			int		t;
 
 			for ( t = 0; t < mi->mi_ntargets; t++ ) {
-				char		**urls = ldap_str2charray( mi->mi_targets[ t ].mt_uri, " " );
+				char		**urls = ldap_str2charray( mi->mi_targets[ t ]->mt_uri, " " );
 				int		u;
 
 				for ( u = 0; urls[ u ] != NULL; u++ ) {
