@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.41.2.17 2007/01/02 21:44:08 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.41.2.23 2008/02/11 23:24:25 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2007 The OpenLDAP Foundation.
+ * Copyright 2003-2008 The OpenLDAP Foundation.
  * Portions Copyright 2003 IBM Corporation.
  * Portions Copyright 2003 Symas Corporation.
  * All rights reserved.
@@ -417,7 +417,6 @@ strings_containment(struct berval* stored, struct berval* incoming)
 static int
 substr_containment_substr(Operation *op, Filter* stored, Filter* incoming)
 {
-	int i;
 	int rc = 0;
 
 	struct berval init_incoming;
@@ -608,7 +607,6 @@ static void
 free_query (CachedQuery* qc)
 {
 	Query* q = (Query*)qc;
-	int i;
 
 	free(qc->q_uuid.bv_val);
 	filter_free(q->filter);
@@ -1025,6 +1023,19 @@ cache_entries(
 	return return_val;
 }
 
+static int
+pcache_op_cleanup( Operation *op, SlapReply *rs ) {
+	slap_callback	*cb = op->o_callback;
+	struct search_info *si = cb->sc_private;
+	if ( si->query.save_attrs != NULL ) {
+		rs->sr_attrs = si->query.save_attrs;
+		op->ors_attrs = si->query.save_attrs;
+	}
+	op->o_callback = op->o_callback->sc_next;
+	op->o_tmpfree( cb, op->o_tmpmemctx );
+	return SLAP_CB_CONTINUE;
+}
+
 static int
 pcache_response(
 	Operation	*op,
@@ -1039,7 +1050,6 @@ pcache_response(
 	if ( si->query.save_attrs != NULL ) {
 		rs->sr_attrs = si->query.save_attrs;
 		op->ors_attrs = si->query.save_attrs;
-		si->query.save_attrs = NULL;
 	}
 
 	if ( rs->sr_type == REP_SEARCH ) {
@@ -1097,8 +1107,7 @@ pcache_response(
 			filter_free( si->query.filter );
 		}
 
-		/* free self */
-		op->o_callback->sc_cleanup = slap_freeself_cb;
+		op->o_callback->sc_cleanup = pcache_op_cleanup;
 	}
 	return SLAP_CB_CONTINUE;
 }
@@ -1125,8 +1134,8 @@ add_filter_attrs(
 		count++;
 	}
 
-	*new_attrs = (AttributeName*)ch_malloc((count+1)*
-		sizeof(AttributeName));
+	*new_attrs = (AttributeName*)ch_calloc( count + 1,
+		sizeof(AttributeName) );
 	for (i=0; i<attrs->count; i++) {
 		(*new_attrs)[i].an_name = attrs->attrs[i].an_name;
 		(*new_attrs)[i].an_desc = attrs->attrs[i].an_desc;
@@ -1146,18 +1155,13 @@ add_filter_attrs(
 			continue;
 		(*new_attrs)[j].an_name = filter_attrs[i].an_name;
 		(*new_attrs)[j].an_desc = filter_attrs[i].an_desc;
-		(*new_attrs)[j].an_oc = NULL;
-		(*new_attrs)[j].an_oc_exclude = 0;
 		j++;
 	}
 	if ( addoc ) {
 		(*new_attrs)[j].an_name = slap_schema.si_ad_objectClass->ad_cname;
 		(*new_attrs)[j].an_desc = slap_schema.si_ad_objectClass;
-		(*new_attrs)[j].an_oc = NULL;
-		(*new_attrs)[j].an_oc_exclude = 0;
 		j++;
 	}
-	BER_BVZERO( &(*new_attrs)[j].an_name );
 }
 
 /* NOTE: this is a quick workaround to let pcache minimally interact
@@ -1205,8 +1209,6 @@ pcache_op_search(
 	cache_manager *cm = on->on_bi.bi_private;
 	query_manager*		qm = cm->qm;
 
-	int count;
-
 	int i = -1;
 
 	AttributeName	*filter_attrs = NULL;
@@ -1219,7 +1221,6 @@ pcache_op_search(
 	int 		cacheable = 0;
 	int		fattr_cnt=0;
 	int		fattr_got_oc = 0;
-	int		oc_attr_absent = 1;
 
 	struct berval tempstr;
 
@@ -1257,7 +1258,7 @@ pcache_op_search(
 				continue;
 			cacheable = 1;
 			template_id = i;
-			Debug( LDAP_DEBUG_NONE, "Entering QC, querystr = %s\n",
+			Debug( pcache_debug, "Entering QC, querystr = %s\n",
 			 		op->ors_filterstr.bv_val, 0, 0 );
 			answerable = (*(qm->qcfunc))(op, qm, &query, i);
 
@@ -2014,7 +2015,7 @@ pcache_db_close(
 	slap_overinst *on = (slap_overinst *)be->bd_info;
 	cache_manager *cm = on->on_bi.bi_private;
 	query_manager *qm = cm->qm;
-	int i, j, rc = 0;
+	int i, rc = 0;
 
 	/* cleanup stuff inherited from the original database... */
 	cm->db.be_limits = NULL;
