@@ -1,8 +1,8 @@
 /* translucent.c - translucent proxy module */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/translucent.c,v 1.1.2.13 2007/01/02 21:44:09 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/translucent.c,v 1.1.2.16 2008/02/11 23:24:25 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2007 The OpenLDAP Foundation.
+ * Copyright 2004-2008 The OpenLDAP Foundation.
  * Portions Copyright 2005 Symas Corporation.
  * All rights reserved.
  *
@@ -540,8 +540,6 @@ static int translucent_search_cb(Operation *op, SlapReply *rs) {
 */
 
 static int translucent_search(Operation *op, SlapReply *rs) {
-	Operation nop = *op;
-
 	slap_overinst *on = (slap_overinst *) op->o_bd->bd_info;
 	slap_callback cb = { NULL, NULL, NULL, NULL };
 	overlay_stack *ov = on->on_bi.bi_private;
@@ -553,11 +551,11 @@ static int translucent_search(Operation *op, SlapReply *rs) {
 	cb.sc_response = (slap_response *) translucent_search_cb;
 	cb.sc_private = private;
 
-	cb.sc_next = nop.o_callback;
-	nop.o_callback = &cb;
+	cb.sc_next = op->o_callback;
+	op->o_callback = &cb;
 
 	op->o_bd->be_private = ov->private;
-	rc = ov->info->bi_op_search(&nop, rs);
+	rc = ov->info->bi_op_search(op, rs);
 	op->o_bd->be_private = private;
 
 	return(rs->sr_err);
@@ -686,7 +684,7 @@ static int translucent_db_init(BackendDB *be) {
 	overlay_stack *ov;
 	int rc;
 
-	Debug(LDAP_DEBUG_TRACE, "==> translucent_init\n", 0, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "==> translucent_db_init\n", 0, 0, 0);
 
 	ov = ch_calloc(1, sizeof(overlay_stack));
 	ov->config = ch_calloc(1, sizeof(translucent_configuration));
@@ -726,11 +724,11 @@ static int translucent_db_open(BackendDB *be) {
 
 	/* "should never happen" */
 	if(!ov->info) {
-		Debug(LDAP_DEBUG_ANY, "translucent_open() called with bad ov->info\n", 0, 0, 0);
+		Debug(LDAP_DEBUG_ANY, "translucent_db_open() called with bad ov->info\n", 0, 0, 0);
 		return(LDAP_OTHER);
 	}
 
-	Debug(LDAP_DEBUG_TRACE, "translucent_open\n", 0, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "translucent_db_open\n", 0, 0, 0);
 
 	be->be_private = ov->private;
 	rc = ov->info->bi_db_open ? ov->info->bi_db_open(be) : 0;
